;;
;; ENTITY MANAGER
;;

.include "man/entityman.h.s"

entityman_set4destruction:
    ld e_cmp(ix),#e_cmp_dead
ret

entityman_resetEntities:
    ld hl,#entity_array
    ld (last_ptr_entity),hl

    ld a,#entity_size * #max_entities
    clean_next_byte:
        ld (hl),#0
        inc hl
        dec a
    jr nz,clean_next_byte

    ld a,#0
    ld (num_entities),a
ret

;; Devuelve un puntero al primer elemento del vector
entityman_getEntityVector_IX:
    ld ix, #entity_array
    ret

;; Devuelve el número de entidades
entityman_getNumEntities_A:
    ld a, (num_entities)
    ret

;; Añade una entidad al array de entidades con los datos pasados por parametro
;; PARAMETROS
;; HL -> Puntero hacia los datos de inicialización
;; REGISTROS USADOS: HL, DE, BC, A
entityman_create:
    ;; Copiar de hl a last_ptr_entity los datos
    ld de, (last_ptr_entity)
    ld bc, #entity_size
    ldir

    ;; Incrementar el num de entidades
    ld a, (num_entities)
    inc a
    ld (num_entities), a

    ;; Apuntar hacia el siguiente espacio libre del array
    ld hl, (last_ptr_entity)
    ld bc, #entity_size
    add hl, bc
    ld (last_ptr_entity), hl

    ret

;; Destruye una entidad
;; PARAMETROS
;; IX -> Entidad a destruir
;; REGISTROS USADDOS: A, HL, DE, BC
entityman_destroy_entity:
    ;; Cargar el último elemento del array de entidades
    ld hl, (last_ptr_entity)
    ld bc, #-entity_size
    add hl, bc
    ld (last_ptr_entity), hl

    ;; Cargamos en de el registro ix
    push ix
    pop de  

    ;; Cambiamos la última entidad por la entidad destruida
    ld bc, #entity_size  ;; Tamaño
    call cpct_memcpy_asm ;; Copiamos el 

    ;; Decrementamos el número de entidades
    ld a,(num_entities)
    dec a
    ld (num_entities), a

    ;; Cargamos otra vez el puntero a la función
    ld hl, #entityman_destroy_entity

    ret


;; Destruye todas las entidades marcadas con el tipo e_cmp_dead
;; PARAMETROS
;; A -> Número de elementos del vector
;; IX -> Puntero al primer elemento del vector
;; REGISTROS USADDOS: AF, HL, DE, BC
entityman_destroy:
    call entityman_destroyDeads
    ret

entityman_destroyDeads:

    destroyDeads_loop:
        
        push af
        ;; Comprueba si la entidad esta muerta
        ld a, e_cmp(ix)
        and #e_cmp_dead
        jr nz,destroy   ;;Si esta muerta

        ;; Apuntamos al siguiente elemento del array
        ld bc, #entity_size
        add ix, bc
        jr not_destroy

        ;; Destruimos la entidad
        destroy:
        call entityman_destroy_entity
        
        not_destroy:
        pop af

        ;; Condición de salida cuando a es 0
        dec a
        jr nz, destroyDeads_loop

    ret z 

;; Actualizamos la fisica de la entidades
;; PARAMETROS
;; IX -> Puntero al primer elemento del vector
;; A -> Número de elementos del vector
;; HL -> Puntero al método a utilizar
;; REGISTROS USADOS: ix, af, hl 
entityman_doForAll:
    ld (metodo), hl

    doForAll_loop:

        push af
        push hl
        ;; Llamamos a la función a llamar
        metodo = . + 1
        call entityman_create

        pop hl
        pop af
        ld (metodo), hl

        ;; Condición de salida cuando a es 0
        dec a
        ret z 

        ;; Apuntamos al siguiente elemento del array
        ld bc, #entity_size
        add ix, bc
        jr doForAll_loop