;; 
;; BULLET MANAGER
;; 

.include "man/spawnman.h.s"

free_balas_mainShip: .db #max_balas_mainShip
rest_time_toShoot_mainShip: .db #t_between_balas
time_create_next_ent: .db 0

spawnman_update_time_shoots:
    ;;Bajar tiempo restante para disparar
    ld a,(rest_time_toShoot_mainShip)
    and a
    jr z,tiempo0    ;;Si el tiempo ya es 0, no descontar

    dec a
    ld (rest_time_toShoot_mainShip),a
    ret nz

    tiempo0:
    ;;Si acaba el tiempo entre balas y faltan balas por recargar resetear tiempo e incrementar balas libres
    ld a,(free_balas_mainShip)
    sub a,#max_balas_mainShip
    ret z

        ;;Aumentar balas
    ld a,(free_balas_mainShip)
    inc a
    ld (free_balas_mainShip),a

        ;;Resetear tiempo
    ld a,#t_between_balas
    ld (rest_time_toShoot_mainShip),a

    ret


;; Crea una entidad en base a la posición de un spawn
;; PARAMETROS
;; HL -> Puntero hacia los datos de inicialización de la entidad a crear
;; IX -> Puntero hacia el spawner (la entidad que dispara)
;; BC -> X e Y adicional que se suman a la coordenadas del spawn. Por ejemplo, si el spawn es (4,4) y bc es (1,0) pues l
;; posicion de la entidad creada sería la suma, es decir, (5, 4)
;; REGISTROS USADOS: HL, DE, BC, A, IY
spawnman_generate_entity::
    push bc
    ;; Copiamos los datos de la entidad a crear
    ld de, (ptr_entity)
    ld bc, #entity_size
    ldir 

    pop bc
    ;; Lo cargamos en IY para mejor manejo 
    ld iy, (ptr_entity)

    ;;Comprobamos si lo crea un enemigo
    dec a
    jr nz,no_enemy

    ;;Asignamos la entidad como enemiga
    ld a, e_cmp2(iy)
    or #e_cmp2_badEntity
    ld e_cmp2(iy),a
    jr continueSpawn

    no_enemy:
    ;;Comprobamos si esta activo el doble_shoot
    ld a, (double_shoot_time)
    and a
    jr z,continueSpawn  ;;Si no esta activo
    
    ;;---------------Double shoot activo-------------------------------
    ;; Posicionamos la entidad creada
    ld a, e_x(ix)
    ld b,#hero_padding
    add b
    ld e_x(iy), a

    ld a, e_y(ix)
    add c
    push bc
    ld e_y(iy), a

    ld hl, (ptr_entity)
    call entityman_create

    ;; Posicionamos la entidad creada
    pop bc
    ld a, e_x(ix)
    ld b, e_w(ix)
    add b
    ld b,#hero_padding
    sub b
    ld e_x(iy), a

    ld a, e_y(ix)
    add c
    ld e_y(iy), a

    ld hl, (ptr_entity)
    call entityman_create

    ret
    ;;---------------------------------------------------------------
    continueSpawn:
    ;; Posicionamos la entidad creada
    ld a, e_x(ix)
    add b
    ld e_x(iy), a

    ld a, e_y(ix)
    add c
    ld e_y(iy), a

    ld hl, (ptr_entity)
    call entityman_create

    ret

;; Genera una entidad para un nivel y le añade el tipo de nivel
;; Cuando esta entidad es destruida se restará 1 a la variable num_lentities
;; PARAMETROS:
;; BC -> X, Y de la entidad a crear
;; DE -> Patrol a asignar
;; HL -> Entidad a crear
spawnman_generate_entity_from_level::
    push bc 
    push de
    ;; Copiamos los datos de la entidad a crear
    ld de, (ptr_entity)
    ld bc, #entity_size
    ldir 
    pop de
    pop bc

    ;; Cargamos la entidad en ix para trabajar más fácil
    ld ix, (ptr_entity)
    
    ;; Asiganmos la coordenadas correspondientes
    ld e_x(ix), b
    ld e_y(ix), c
    ld e_ai_c_patrol_h(ix), d
    ld e_ai_c_patrol_l(ix), e

    ;; Asiganmos el tipo que nos indican si han sido creados por un nivel
    ld a, e_cmp(ix)
    xor #e_cmp_level
    ld e_cmp(ix), a

    ;; Añadimos la entidad al array de entidades
    ld hl, (ptr_entity)
    call entityman_create

    ret

;; Genera una entidad desde la posición del jugador
;; PARÁMETROS
;; HL -> Entidad a generar
;; IX -> Hero
;; REGISTROS USADOS: HL, DE, BC, A, IY
spawnman_hero:
    ;; Comprobamos que se puede generar
    ld a,(free_balas_mainShip)
    and a
    ret z

    restaBala: 
        ;;Restamos uno a las balas libres
        dec a
        ld (free_balas_mainShip),a

    ld a, e_w(ix)   ;;a=ancho
    rra             ;;Desplazamiento derecha de a
    and #0b01111111 ;;a=anho/2

    ld b, a        ;;Mitad del ancho
    
    ld iy,#bullet
    ld a, e_h(iy)   ;;a=alto de la bala
    xor #0xFF
    inc a           ;;a=-alto
    ld c,a

    call spawnman_generate_entity
    ret

;; Genera una entidad desde la posición del enemigo
;; PARÁMETROS
;; HL -> Entidad a generar
;; IX -> Hero
;; REGISTROS USADOS: HL, DE, BC, A, IY
spawnman_enemy:
    ld a, e_w(ix)   ;;a=ancho
    rra             ;;Desplazamiento derecha de a
    and #0b01111111 ;;a=anho/2

    ld b, a        ;;Mitad del ancho
    ld c, e_h(ix)   ;;Alto
    ld a,#1         ;; Llamar a spawnman diciendo que lo crea un enemigo
    call spawnman_generate_entity
    ret

;; Genera una entidad en una posición (rand, 1)
;; PARÁMETROS
;; HL -> Entidad a generar
;; REGISTROS USADOS: HL, DE, BC, A, IY
spawnman_rand_x:
    ld ix, #empty
    push hl

    call cpct_getRandom_lcg_u8_asm
    and #63    
    
    ;; Guardamos el número aleatorio en b
    ld b, a
    ld c, #1 
    ld a, #1
    pop hl
    call spawnman_generate_entity
    ret

;; Selección del enemigo aleatorio que cae
select_rand_entity_to_create:
    ;; Elegimos un número entre 0 - 128
    call cpct_getRandom_lcg_u8_asm
    and #127

    ;; Cargamos las entidades posibles a crear
    ld ix, #spawner
    ld bc, #3

    ;; En cada iteracción vamos restando la probabilidad al número al azar
    ;; hasta que salga negativo. Entonces elegimos esa entidad a crear
    loop_select:
        sub 0(ix)
        jr c, end_select
        add ix, bc
        jr loop_select

    ;; Cargamos la entidad a crear
    end_select:
    ld l, 1(ix)
    ld h, 2(ix)
    ret

;; Spawnea una entidad desde arriba cada un determinado tiempo marcado por time_create_next_ent
spawnman_rand_entity_update::
    
    ;;Bajar tiempo restante para generar asteroide
    ld a,(time_create_next_ent)
    or a
    jr z,t0    ;;Si el tiempo ya es 0, no descontar

    ;; Decrementamos el tiempo y comprobamos si == 0
    dec a
    ld (time_create_next_ent),a
    ret nz

    ;; Seleccionar y crear la entidad aleatoria
    t0:
    call select_rand_entity_to_create
    call spawnman_rand_x

    ;;Resetear tiempo
    call cpct_getRandom_lcg_u8_asm
    and #127
    ld (time_create_next_ent), a

    ret