;;
;; COLLISION SYSTEM
;;

.include "man/entityman.h.s"
.include "man/heroman.h.s"

;; Compara si dos entidades colisionan
;; PARAMETROS
;; IX -> Entidad 1
;; IY -> Entidad 2
collisionsys_compare::
    ;;jr .
    ;;Comprobamos si IX es player/hero
    ld a, e_cmp(ix)
    and #e_cmp_hero
    jr nz, test_collision

    ;; Comprobamos si es colisionable IX
    ld a, e_cmp(ix)
    and #e_cmp_col
    ret z

    ;; Comprobamos que no es una entidad muerta
    ld a, e_cmp(ix)
    and #e_cmp_dead
    ret nz

    test_collision:

    push ix
    push iy
    pop bc
    pop de
    ld a,d
    sub a,b
    jr nz,not_same
    ld b,c
    ld a,e
    sub a,b
    ret z

    not_same:
    ;;Killer por encima (y_killer + altura_killer <= y_colisionable)
    ld c,e_y(ix) ;;y colisionable
    ld b,e_y(iy) ;;y killer
    ld a,e_h(iy) ;;altura killer

    add a,b
    sub a,c

    ret m
    ret z
    ;;Killer por debajo (y_colisionable + altura_colisionable <= y_killer)
    ld a,e_y(ix) ;;y colisionable
    ld c,e_y(iy) ;;y killer
    ld b,e_h(ix) ;;altura colisionable

    add a,b
    sub a,c

    ret m
    ret z

    ;;Comprobar si se colisiona con player/hero
    ld a, e_cmp(ix)
    and #e_cmp_hero
    jr z, no_hero_col_calculate    ;;Si no es hero

    ;;HERO
    ld e,#hero_padding
    jr continueCalc

    ;;NO_HERO
    no_hero_col_calculate: ld e,#0

    continueCalc:
    ;;Killer por la izquierda (x_killer + ancho_killer <= x_colisionable + e)
    ld a,e_x(ix) ;;x colisionable
    ld b,e
    add a,b
    ld c,a  ;;x colisionable + e

    ld b,e_x(iy) ;;x killer
    ld a,e_w(iy) ;;ancho killer

    add a,b
    sub a,c

    ret m
    ret z

    ;;Killer por la derecha (x_colisionable + ancho_colisionable - e <= x_killer)
    ld a,e_w(ix) ;;ancho colisionable
    ld b,e
    sub b
    ld b,a       ;;ancho colisionable - e

    ld a,e_x(ix) ;;x colisionable
    ld c,e_x(iy) ;;x killer
    
    add a,b
    sub a,c

    ret m
    ret z

    ;;Colision
    ;;Comprobar si el killer (iy) ya habia colisionado
    ld a,e_cmp(iy)
    and #e_cmp_dead
    ret nz

    ;;Comprobar si el colisionable (ix) se trata de nave enemiga (e_cmp_level)
    ld a, e_cmp(ix)
    and #e_cmp_level
    jr nz, enemyship

    ld a, e_cmp2(ix)
    and #e_cmp2_badEntity

    jr nz, enemyship

    call no_cmp_level
    
    enemyship:
    ;;NAVE ENEMIGA
    ;;Comprobamos si la bala es enemiga
    ld a,e_cmp2(iy)
    and #e_cmp2_badEntity
    ret nz   ;;Si la bala es enemiga

    ld e_cmp(iy), #e_cmp_dead   ;;Matamos la bala
    ld e_cmp2(iy), #e_cmp2_default   ;;Matamos la bala

    ;;Restamos 1 vida   
    ld a,e_lives(ix)
    dec a
    ld e_lives(ix),a
    ret nz
    ;;Si no le quedan vidas muere
    ld a, e_cmp(ix)
    and #e_cmp_level
    call nz, dec_level  ;;Bajamos 1 al contador de naves restantes

    ld a, e_score(ix)
    ld b,#0
    ld c,a  ;;bc = puntuacion a sumar
    call add_score ;;Sumamos puntuacion
    
    no_cmp_level:
    ;;Comprobar si se colisiona con player/hero
    ld a, e_cmp(ix)
    and #e_cmp_hero
    jr nz, hero    ;;Si es hero
    
    ld e_cmp(ix), #e_cmp_dead
    ld e_cmp2(ix), #e_cmp2_default

    ld hl, #explosion
    ld a,#1
    ld b,#0
    ld c,#0
    call spawnman_generate_entity   

    ;; Generamos la explosión
    ret 

    ;;HERO
    hero: 
        ;;Comprobar si se trata del double_score
        ld a, e_cmp2(iy)
        and #e_cmp2_doubleScore0
        jr z,no_double_score

        ;;DOUBLE SCORE
        ld e_cmp(iy), #e_cmp_dead
        ld e_cmp2(iy), #e_cmp2_default
        call active_double_score
        ret

        ;;Comprobar si se trata de un corazon
        no_double_score:
        ld a, e_cmp2(iy)
        and #e_cmp2_heart0
        jr z,no_corazon

        ;;CORAZON
        ld e_cmp(iy), #e_cmp_dead
        ld e_cmp2(iy), #e_cmp2_default
        call heroman_addHeart
        ret

        ;;Comprobar si se trata del doble_shoot
        no_corazon:
        ld a, e_cmp2(iy)
        and #e_cmp2_doubleShoot0
        jr z,no_doble_shoot

        ;;DOBLE SHOOT
        ld e_cmp(iy), #e_cmp_dead
        ld e_cmp2(iy), #e_cmp2_default
        call heroman_activeDoubleShoot
        ret

        ;;OBJETO DAÑINO
        no_doble_shoot:

        ld e_cmp(iy), #e_cmp_dead
        ld e_cmp2(iy), #e_cmp2_default
        call hero_collision
    
    ret

dec_level:
    ld a, (num_lentities)
    dec a
    ld (num_lentities), a
    ret

; collisionsys_calculateBoxColliderHero:
;     ;; x = x + 4
;     ld a, e_x(ix)
;     add b
;     ld e_x(ix), a
;     ;; x = x + 4

;     ld a, e_y(ix)
;     add c
;     ld e_y(ix), a

;     ld a, b
;     add a 
;     neg
;     ld b, a
;     ld a, e_w(ix)
;     add b
;     ld e_w(ix), a

;     ld a, c
;     add a
;     neg
;     ld b, a
;     ld a, e_h(ix)
;     add b
;     ld e_h(ix), a

;     ret

collisionsys_entity::

    ;; Comprobamos si IX es colisionable
    ld a, e_cmp2(ix)
    and #e_cmp2_killer
    ret z

    ;; Comprobamos que no es una entidad muerta
    ld a, e_cmp(ix)
    and #e_cmp_dead
    ret nz

    ;; Pones ix en iy para poder utilizar el bucle doForAll que sobreescribe el ix
    ;; IY = IX
    push ix 
    pop iy
    push ix
    
    ;; Comparamos IY con el resto
    call entityman_getEntityVector_IX
    call entityman_getNumEntities_A
    ld hl, #collisionsys_compare
    call entityman_doForAll

    ;; Devolvemos los valores temporales
    pop ix

    ret

collisionsys_update::
    ;ld b, #2
    ;ld c, #2
    ;call collisionsys_calculateBoxColliderHero

    ld hl, #collisionsys_entity
    call entityman_doForAll

    call entityman_getEntityVector_IX
    
    ;ld b, #-2
    ;ld c, #-2
    ;call collisionsys_calculateBoxColliderHero

    ret