;;
;; Physics render
;; 
.include "man/entityman.h.s"

;; Constantes
screen_height == 200
screen_width  == 79

;; Actualiza la posición y teniendo en cuenta la altura de la pantalla
;; PARAMETROS
;; IX -> Puntero a la entidad a comprobar
;; REGISTROS USADOS: A, C
physicssys_checkCollision_y::
    ;; Cargamos altura de la pantalla y restamos la altura del sprite
    ld a, #screen_height + 1 ;; A = 201
    sub e_h(ix)              ;; A = 201 - Height
    ld c, a                  ;; C = A (201 - Height)

    ;; Comprobamos si esta en una posición válida
    ld a, e_y(ix) ;; A = Y 
    cp c          ;; A > C || A < 0? (Y > 201 - Height || Y < 0)
    ret c         ;; return

    ;; Si no esta en una posición válida se marca como muerta
    ld a, e_cmp(ix)
    xor #e_cmp_dead
    ld e_cmp(ix), a
    ld e_cmp2(ix), #e_cmp2_default

    ret

;; Actualiza la posición y teniendo en cuenta la anchura de la pantalla
;; PARAMETROS
;; IX -> Puntero a la entidad a comprobar
;; REGISTROS USADOS: A, C
physicssys_checkCollision_x::
    ;; Cargamos anchura de la pantalla y restamos la anchura del sprite
    ld a, #screen_width + 1 ;; A = 81
    sub e_w(ix)              ;; A = 81 - Width
    ld c, a                  ;; C = A (81 - Width)

    ;; Comprobamos si esta en una posición válida
    ld a, e_x(ix) ;; A = Y 
    cp c          ;; A > C || A < 0? (Y > 81 - Width || Y < 0)
    ret c         ;; return

    ;; Si no esta en una posición válida rebota
    ld a, e_vx(ix)
    neg
    ld e_vx(ix), a

    ld a, e_x(ix)
    add e_vx(ix)   
    ld e_x(ix), a 

    ret

;; Actualizar la física de una entidad 
;; PARAMETROS
;; IX -> Puntero a la entidad a actualizar
;; REGISTROS USADOS: IX, A
physicssys_update_entity::
    ;; Si es de tipo explosión
    ;; decremento el contador
    ld a, e_cmp2(ix)
    and #e_cmp2_explosion
    jr z, continue_update_phys

    ld a, e_ai_shoot(ix)
    or a
    jr nz, dec_live_time

    ld e_cmp(ix), #e_cmp_dead

    dec_live_time:
    dec a
    ld e_ai_shoot(ix), a

    continue_update_phys:
    ;; Actualizar x
    ld a, e_x(ix)
    add e_vx(ix)   
    ld e_x(ix), a 

    ;; Actualizar y
    ld a, e_y(ix)
    add e_vy(ix)   
    ld e_y(ix), a 

    ;; Si es de tipo e_cmp_dead_y
    ;; Comprobar colision en los bordes de y
    ld a, e_cmp(ix)
    and #e_cmp_dead_y
    call nz, physicssys_checkCollision_y

    ;; Si es de tipo e_cmp_bounce_x
    ;; Rebota en los bordes del eje x
    ld a, e_cmp(ix)
    and #e_cmp_bounce_x
    call nz, physicssys_checkCollision_x

    ld a, e_cmp(ix)
    and #e_cmp_hero
    call nz, update_ap

    ret

update_ap:
    ld a, e_h(ix)
    sub #36
    ret nc

    ld a, e_h(ix)
    add #2
    ld e_h(ix), a

    ld a, e_y(ix)
    add #-2
    ld e_y(ix), a

    ret

;; Actualizamos la fisica de la entidades
;; PARAMETROS
;; IX -> Puntero al primer elemento del vector
;; A -> Número de elementos del vector
;; REGISTROS USADOS: IX, A, DE, HL
physicssys_update::
    ld hl, #physicssys_update_entity
    call entityman_doForAll
    ret